package cn.timer.api.controller.zpgl;

import cn.timer.api.aspect.lang.annotation.Log;
import cn.timer.api.aspect.lang.enums.BusinessType;
import cn.timer.api.bean.disk.DiskCatalogue;
import cn.timer.api.bean.yggl.YgglMainEmp;
import cn.timer.api.bean.zpgl.ZpglFail;
import cn.timer.api.bean.zpgl.ZpglMslcLog;
import cn.timer.api.bean.zpgl.ZpglRcxx;
import cn.timer.api.bean.zpgl.ZpglZwxx;
import cn.timer.api.config.annotation.CurrentUser;
import cn.timer.api.config.annotation.UserBean;
import cn.timer.api.config.enuminterface.JxglEnumInterface;
import cn.timer.api.config.exception.CustomException;
import cn.timer.api.controller.zpgl.sevice.ZpglMslcLogService;
import cn.timer.api.dao.zpgl.ZpglMslcLogMapper;
import cn.timer.api.dto.zpgl.ZpglRcxxDto;
import cn.timer.api.utils.Result;
import cn.timer.api.utils.ResultUtil;
import cn.timer.api.utils.ZpglMessageTemplate;
import cn.timer.api.utils.query.BaseQuery;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.transaction.Transactional;
import java.text.MessageFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 招聘管理
 *
 * @author wuqingjun
 * @email 284718418@qq.com
 * @date 2022-03-08 15:14:40
 */
@Api(tags = "招聘管理")
@Transactional(rollbackOn = Exception.class)
@RestController
@RequestMapping(value = "/zpgl", produces = {"application/json"})
public class ZpglController {

    @Autowired
    private ZpglMslcLogMapper zpglMslcLogMapper;
    @Autowired
    private ZpglMslcLogService zpglMslcLogService;

    /**
     * 招聘-招聘中
     *
     * @param
     * @return
     */
    @GetMapping(value = "/recruit")
    @ApiOperation(value = "1.招聘列表", httpMethod = "GET", notes = "招聘列表")
    @ApiOperationSupport(order = 1)
    public Result<Object> recruit(@CurrentUser UserBean userBean, BaseQuery baseQuery,
                                  @ApiParam("搜索关键字") @RequestParam(required = false) String query,
                                  @ApiParam("职位ID") @RequestParam(required = false) Integer zpglZwxxId,
                                  @ApiParam("状态") @RequestParam(required = false) Integer status) {
        Page<ZpglRcxx> page = new Page<ZpglRcxx>(baseQuery.getPageNum(), baseQuery.getPageSize());
        QueryWrapper<ZpglRcxx> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ZpglRcxx::getOrgCode, userBean.getOrgCode());
        if (!StringUtils.isEmpty(zpglZwxxId) && zpglZwxxId > 0) {
            queryWrapper.lambda().eq(ZpglRcxx::getZpglZwxxId, zpglZwxxId);
        }
        if (!StringUtils.isEmpty(query)) {
            queryWrapper.lambda().and(qw -> qw.like(ZpglRcxx::getName, query).or().eq(ZpglRcxx::getMobile, query).or().eq(ZpglRcxx::getMail, query));
        }
        queryWrapper.lambda().eq(ZpglRcxx::getStatus, status);
        IPage<ZpglRcxx> ygglMainEmpPage = ZpglRcxx.builder().build().selectPage(page, queryWrapper);
        List<ZpglRcxx> zpglRcxxs = page.getRecords();
        ygglMainEmpPage.getCurrent();
        ygglMainEmpPage.getPages();
        ygglMainEmpPage.getTotal();
        ygglMainEmpPage.getSize();
        Map<String, Object> map = new HashMap<>(2);
        map.put("zpglRcxxs", zpglRcxxs);
        map.put("FILTERING", ZpglRcxx.builder().build().selectCount(new QueryWrapper<ZpglRcxx>().lambda().eq(ZpglRcxx::getStatus, JxglEnumInterface.ResumeStatus.FILTERING.getType())));
        map.put("FILTER_PASS", ZpglRcxx.builder().build().selectCount(new QueryWrapper<ZpglRcxx>().lambda().eq(ZpglRcxx::getStatus, JxglEnumInterface.ResumeStatus.FILTER_PASS.getType())));
        map.put("INTERVIEW", ZpglRcxx.builder().build().selectCount(new QueryWrapper<ZpglRcxx>().lambda().eq(ZpglRcxx::getStatus, JxglEnumInterface.ResumeStatus.INTERVIEW.getType())));
        map.put("INTERVIEWED", ZpglRcxx.builder().build().selectCount(new QueryWrapper<ZpglRcxx>().lambda().eq(ZpglRcxx::getStatus, JxglEnumInterface.ResumeStatus.INTERVIEWED.getType())));
        map.put("INTERVIEW_PASS", ZpglRcxx.builder().build().selectCount(new QueryWrapper<ZpglRcxx>().lambda().eq(ZpglRcxx::getStatus, JxglEnumInterface.ResumeStatus.INTERVIEW_PASS.getType())));
        map.put("EMPLOY", ZpglRcxx.builder().build().selectCount(new QueryWrapper<ZpglRcxx>().lambda().eq(ZpglRcxx::getStatus, JxglEnumInterface.ResumeStatus.EMPLOY.getType())));
        map.put("OFFER", ZpglRcxx.builder().build().selectCount(new QueryWrapper<ZpglRcxx>().lambda().eq(ZpglRcxx::getStatus, JxglEnumInterface.ResumeStatus.OFFER.getType())));
        map.put("ENTRYING", ZpglRcxx.builder().build().selectCount(new QueryWrapper<ZpglRcxx>().lambda().eq(ZpglRcxx::getStatus, JxglEnumInterface.ResumeStatus.ENTRYING.getType())));
        return ResultUtil.data(ygglMainEmpPage, map, "搜索成功");
    }

    /**
     * 招聘-招聘中
     *
     * @param
     * @return
     */
    @GetMapping(value = "/zwxx")
    @ApiOperation(value = "2.职位信息", httpMethod = "GET", notes = "职位信息")
    @ApiOperationSupport(order = 2)
    public Result<Object> zwxx(@CurrentUser UserBean userBean) {
        QueryWrapper<ZpglZwxx> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ZpglZwxx::getOrgCode, userBean.getOrgCode());
        List<ZpglZwxx> zpglZwxxs = ZpglZwxx.builder().build().selectList(queryWrapper);
        return ResultUtil.data(zpglZwxxs, "查询成功");
    }

    /**
     * 招聘-变更状态
     *
     * @param
     * @return
     */
    @PostMapping(value = "/operation")
    @ApiOperation(value = "3.招聘-变更状态", httpMethod = "POST", notes = "招聘-变更状态")
    @ApiOperationSupport(order = 3)
    public Result<Object> operation(@CurrentUser UserBean userBean, @RequestBody ZpglRcxxDto zpglRcxxDto) {
        try {
            ZpglRcxx zpglRcxx = ZpglRcxx.builder().id(zpglRcxxDto.getId()).status(zpglRcxxDto.getStatus()).build();
            ZpglMslcLog zpglMslcLog = new ZpglMslcLog();
            zpglMslcLog.setZpglRcxxId(zpglRcxxDto.getId());
            zpglMslcLog.setUserId(userBean.getEmpNum());
            zpglMslcLog.setUserName(userBean.getQyzxEmpLogin().getUsername());
            String detail = "";
            switch (zpglRcxxDto.getStatus()) {
                case 2:
                    //初筛通过 填充字段
                    zpglRcxx.setIfrck(zpglRcxxDto.getIfrck());
                    detail = ZpglMessageTemplate.print(ZpglMessageTemplate.FILTER_PASS, new String[]{
                            userBean.getQyzxEmpLogin().getUsername()});
                    break;
                case 3:
                    //安排面试 填充字段
                    zpglRcxx.setInterviewNum(zpglRcxxDto.getInterviewNum());
                    zpglRcxx.setInterviewWay(zpglRcxxDto.getInterviewWay());
                    zpglRcxx.setInterviewerId(zpglRcxxDto.getInterviewerId());
                    zpglRcxx.setInterviewTime(zpglRcxxDto.getInterviewTime());
                    zpglRcxx.setNoticeWay(zpglRcxxDto.getNoticeWay());
                    YgglMainEmp ygglMainEmp = YgglMainEmp.builder().build().selectOne(new QueryWrapper<YgglMainEmp>().select("name").lambda().eq(YgglMainEmp::getEmpNum, zpglRcxxDto.getInterviewerId()));
                    detail = ZpglMessageTemplate.print(ZpglMessageTemplate.INTERVIEW, new String[]{
                            userBean.getQyzxEmpLogin().getUsername(),
                            zpglRcxxDto.getInterviewNum().toString(),
                            new SimpleDateFormat("yyyy-MM-dd' 'HH:mm:ss").format(zpglRcxxDto.getInterviewTime()),
                            ygglMainEmp.getName()});
                    break;
                case 4:
                    detail = ZpglMessageTemplate.print(ZpglMessageTemplate.INTERVIEWED, new String[]{
                            userBean.getQyzxEmpLogin().getUsername(), zpglRcxxDto.getInterviewNum().toString()});
                    break;
                case 5:
                    detail = ZpglMessageTemplate.print(ZpglMessageTemplate.INTERVIEW_PASS, new String[]{
                            userBean.getQyzxEmpLogin().getUsername()});
                    break;
                case 6:

                    detail = ZpglMessageTemplate.print(ZpglMessageTemplate.EMPLOY, new String[]{
                            userBean.getQyzxEmpLogin().getUsername()});
                    break;
                case 7:
                    //发送offer 填充字段
                    zpglRcxx.setBmgwId(zpglRcxxDto.getBmgwId());
                    zpglRcxx.setJobType(zpglRcxxDto.getJobType());
                    zpglRcxx.setYjrzTime(zpglRcxxDto.getYjrzTime());
                    detail = ZpglMessageTemplate.print(ZpglMessageTemplate.OFFER, new String[]{
                            userBean.getQyzxEmpLogin().getUsername()});
                    //发送offer
                    break;
                case 8:
                    //待入职 填充字段
                    zpglRcxx.setBmgwId(zpglRcxxDto.getBmgwId());
                    zpglRcxx.setJobType(zpglRcxxDto.getJobType());
                    zpglRcxx.setYjrzTime(zpglRcxxDto.getYjrzTime());
                    detail = ZpglMessageTemplate.print(ZpglMessageTemplate.WAIT, new String[]{
                            userBean.getQyzxEmpLogin().getUsername()});
                    break;
                case 9:
                    //淘汰 填充字段
                    zpglRcxx.setZpglFailId(zpglRcxxDto.getZpglFailId());
                    zpglRcxx.setRemarks(zpglRcxxDto.getRemarks());
                    ZpglFail zpglFail = ZpglFail.builder().id(zpglRcxxDto.getZpglFailId()).build().selectById();
                    String messageTemplate = ZpglMessageTemplate.ENTRYING;
                    if (!StringUtils.isEmpty(zpglFail)) {
                        if (zpglFail.getZpglRcxxStatus().equals(JxglEnumInterface.ResumeStatus.INTERVIEW_PASS.getType())) {
                            messageTemplate = ZpglMessageTemplate.INTERVIEWED_ENTRYING;
                        }
                        if (zpglFail.getZpglRcxxStatus().equals(JxglEnumInterface.ResumeStatus.OFFER.getType())) {
                            messageTemplate = ZpglMessageTemplate.OFFER_ENTRYING;
                        }
                        detail = ZpglMessageTemplate.print(messageTemplate, new String[]{
                                userBean.getQyzxEmpLogin().getUsername(), zpglFail.getFailCause(), zpglRcxxDto.getRemarks()});
                    }
                    break;
                case 10:
                    detail = ZpglMessageTemplate.print(ZpglMessageTemplate.ENTRY, new String[]{
                            userBean.getQyzxEmpLogin().getUsername()});
                    break;
                default:
                    break;
            }
            zpglMslcLog.setDetail(detail);
            zpglRcxx.updateById();
            zpglMslcLogService.saveZpglMslcLog(zpglMslcLog);
        } catch (Exception e) {
            e.printStackTrace();
            throw new CustomException("招聘-变更状态失败");
        }
        return ResultUtil.success();
    }

}
